/*** skeleton-main.c v.2.0
     David E. Moriarty
     April 1997

     This file is designed as a starting point for implementing SANE v2.0
     to new domains.
***/

#include "sane.h"
#include "sane-ea.h"
#include "sane-util.h"
#include "sane-nn.h"
#include "sane-critic.h"
#include "sane-seed.h"

#define NUM_ATTEMPTS     8    /*# of times network is evaluated*/
#define RANDOM_POP       1    /*start with random population*/


// global data, defined as externs in sane.h
best_net_structure actual_best_nets[NUM_TRIALS];
best_net_structure *best_nets[NUM_TRIALS];
int generation;               /*global generation counter*/
char savefile[15];            /*what's the name of the population file*/

void eval_pop(neuron**);
void find_champion(neuron**);
float evaluate_net(network*);

int menu(void)
{
   int i;
   printf("\n\n");
   printf("---------- Menu ----------");
   printf("\n   1.  Evaluate subpopulation");
   printf("\n   2.  Create random population");
   printf("\n   3.  Load population file");
   printf("\n   4.  Save population to file"); 
   printf("\n   5.  Load sub-population");
   printf("\n   7.  Find champion");
   printf("\n   9.  Outut Network Vectors for PCA");
   printf("\n\n Choice: ");
   scanf("%d",&i);
   return i;
}

/** eval_pop evaluates entire population to get the average fitness of
  each neuron and the fitness of each network blueprint.  This
  function is called by evolve and should put the fitness of each
  neuron in the neuron's fitness fiels (pop[i]->fitness).
    **/
int main(int argc, char *argv[])
{
  int i;

  neuron actual_pop[POP_SIZE],*pop[POP_SIZE],*npop[NUM_HIDDEN];
  int choice;
  char filename[30];
  network net;

   /*initialize neuron structure*/
   for (i=0;i<POP_SIZE;++i)
      pop[i] = &actual_pop[i];
   for(i=0;i<NUM_HIDDEN;++i)
      npop[i] = pop[i];

   /* to evolve from command line use form:

       a.out <popfile> <generations> <seed> >errorfile & 
      
      to evaluate the best network files use:
     
       a.out x &
  */

   if (argc > 1) {
      /*if second argument is x, we need to test the saved network files*/
      if (argv[1][0] == 'x') {
         find_champion(npop);
         exit(1);
       }
      srand(atoi(argv[3]));
      reseed(atoi(argv[3]));
      if (RANDOM_POP)
        create_pop(pop); 
      else
        load_pop(pop,argv[1]); 
      strcpy(savefile,argv[1]);
      evolve(pop,atoi(argv[2]));
      exit(1);
    }

   /* menu is presented if no arguments are given to main. */

   printf("\n enter a seed: ");
   scanf("%d",&choice);
   srand(choice); 
   reseed(choice);
   while (1) {
      choice = menu();
      switch (choice) {
        case 1: build_net(&net,npop);
                evaluate_net(&net);
                break;
        case 2: create_pop(pop);
                break;
        case 3: printf("\n Enter filename: ");
                scanf("%s",filename);
                load_pop(pop,filename);
                printf("Diversity = %f",Phi(pop));
                break;
        case 4: printf("\n Enter filename: ");
                scanf("%s",filename);
                save_pop(pop,filename); 
                break;
        case 5: printf("\n Enter filename of subpop: ");
                scanf("%s",filename);
                load_partial(pop,filename);
                break;
        case 7: find_champion(npop);
                break;
        case 9: cluster_output(pop,"r-vectors.txt","r-names.txt",POP_SIZE);
                 break;
        default:
                exit(1);
	}
    }
  exit(1);
}


void eval_pop(neuron *pop[])
{
   int i,j,k;
   float pop_average;
   float total_fitness=0.0;
   char best_name[15];
   neuron *net_pop[NUM_HIDDEN];
   network net;
   

  /*make new file name for best network*/
   strcpy(best_name,"000best.bin");
   i = generation%1000;            
   j = i/100;
   best_name[0] = j + 48;
   j = i%100;
   best_name[1] = j/10 + 48;
   best_name[2] = j%10 + 48;

   /*reset fitness and test values of each neuron*/
   for(i=0;i<POP_SIZE;++i)  
     pop[i]->fitness = pop[i]->ranking = pop[i]->tests = 0;

   /*evaluation stage.  Create NUM_TRIALS networks, each containing
     NUM_HIDDEN neurons.  First build networks by following pointers from
     the best_nets structure.  If the TOP_NETS is less than
     NUM_TRIALS, build the remaining networks by randomly selecting
     neurons. */

   for(i=0;i<NUM_TRIALS;++i) {
      if (i < TOP_NETS)
        for(j=0;j<NUM_HIDDEN;++j) {
          net_pop[j] = best_nets[i]->neuron[j];
          ++net_pop[j]->tests;
       }        
      else        /*find random subpopulation*/
        for(j=0;j<NUM_HIDDEN;++j) {         
           net_pop[j] = pop[lrand48()%POP_SIZE];
           best_nets[i]->neuron[j] = net_pop[j];
           ++net_pop[j]->tests;
         }
   
      build_net(&net,net_pop);
      best_nets[i]->fitness = 0.0;
     /*evaluate network*/
      for(j=0;j<NUM_ATTEMPTS;++j) 
        best_nets[i]->fitness += evaluate_net(&net); 
      total_fitness += best_nets[i]->fitness;
    }         
   pop_average = total_fitness/NUM_TRIALS;
   sort_best_networks();

   /*Each neuron's fitness is determined from the best 5 networks it
     participated in.  It may be better to use the best 3.
     */

   for(j=0;j<NUM_TRIALS;++j)
     for(k=0;k<NUM_HIDDEN;++k)
       if (best_nets[j]->neuron[k]->ranking < 5) {
         ++best_nets[j]->neuron[k]->ranking;
         best_nets[j]->neuron[k]->fitness += best_nets[j]->fitness;
       }

   /*save best network to a file*/
   save_partial(best_nets[0]->neuron,best_name);      
   (void) pop_average; /* avoid warning about not being used */
}


/** find_champion will read in the best networks (from files) and
  evaluate them.  The performance of each network will be reported in
  report.txt.  This is designed as way to find the best overall network.
  **/

void find_champion(neuron *npop[])
{
  int i,j,l;
  float fitness;
  float best=-999999;
  char subname[15];
  network net;
  FILE *fptr;

  strcpy(subname,"000best.bin");
  for(l=0;l<1000;++l) {
   j = l/100;
   subname[0] = j + 48;
   j = l%100;
   subname[1] = j/10 + 48;
   subname[2] = j%10 + 48;
   if ((fptr = fopen(subname,"r")) != NULL) {
      fclose(fptr);
      load_partial(npop,subname);
      build_net(&net,npop);
      fitness = 0;
      
      /*evaluate network*/
      for(i=0;i<50;++i)
        fitness += evaluate_net(&net);
      if (fitness > best)
        best = fitness;
      fptr = fopen("report.txt","a");
      fprintf(fptr,"%d\t%f\n",l,best/50.0);
      fclose(fptr);
    }
 }
 
  fptr=fopen("analyze.out","a");
  fprintf(fptr,"%f\n",best/50.0);
  fclose(fptr);  
  printf("%f\n",best/50.0);
}

// sample evaluation: try to have the network return the addition
// of two numbers from 0..0.5
float evaluate_net(network *net)
{
  float correctresult;
  float retval;

  net->input[0] = (rand()&255) / (2.0 * 255);
  net->input[1] = (rand()&255) / (2.0 * 255);
  correctresult = net->input[0] + net->input[1];
  activate_net(net);
  retval = fabs(correctresult - net->sigout[0]);

  return 1.0 - retval;
}

